import { computed, reactive, watch } from 'vue'
import type { Ref } from 'vue'
import Konva from 'konva'

const useStage = (stageRef: Ref<Konva.StageConfig | undefined>, scale: Ref<number>) => {
  const stage = computed<Konva.Stage>(() => stageRef.value?.getNode())
  const imageSize = reactive({ width: 0, height: 0 })

  const configStage = reactive({
    width: window.innerWidth,
    height: window.innerHeight,
    draggable: true,
    dragBoundFunc: (pos: { x: number; y: number }) => {
      const { stageX = 0, stageY = 0 } = stage.value.attrs
      let x = pos.x - stageX
      let y = pos.y - stageY
      const paddingX = imageSize.width * scale.value - configStage.width
      const paddingY = imageSize.height * scale.value - configStage.height
      if (paddingX + x < 0) x = -paddingX
      if (paddingY + y < 0) y = -paddingY
      if (configStage.width + paddingX + x > imageSize.width * scale.value) x = 0
      if (configStage.height + paddingY + y > imageSize.height * scale.value) y = 0
      x += stageX
      y += stageY
      return { x, y }
    }
  })

  const setZoom = () => {
    if (scale.value < 0) {
      return
    }
    const maxScale = Math.min(
      window.innerWidth / imageSize.width,
      window.innerHeight / imageSize.height
    )
    stage.value.scale({ x: scale.value, y: scale.value })
    if (scale.value <= maxScale) {
      configStage.width = imageSize.width * scale.value
      configStage.height = imageSize.height * scale.value
    } else {
      configStage.width = imageSize.width * maxScale
      configStage.height = imageSize.height * maxScale
    }
  }

  watch(scale, () => {
    setZoom()
  })

  return { configStage, imageSize, stage }
}

export default useStage
